package com.yidianling.course.coursePlay

import android.content.Intent
import android.graphics.drawable.Drawable
import android.text.TextUtils
import android.util.Log
import android.view.ViewGroup
import androidx.constraintlayout.widget.ConstraintLayout
import com.alibaba.android.arouter.facade.annotation.Route
import com.bumptech.glide.Glide
import com.bumptech.glide.load.resource.bitmap.RoundedCorners
import com.bumptech.glide.load.resource.drawable.DrawableTransitionOptions
import com.bumptech.glide.request.RequestOptions
import com.bumptech.glide.request.target.ViewTarget
import com.bumptech.glide.request.transition.Transition
import com.ydl.media.audio.AudioPlayer
import com.ydl.media.audio.model.Music
import com.ydl.media.view.PlayTypeEnum
import com.ydl.media.view.PlayerFloatHelper
import com.ydl.webview.H5Params
import com.ydl.webview.NewH5Activity
import com.ydl.ydl_image.module.GlideApp
import com.ydl.ydlcommon.base.BaseMvpActivity
import com.ydl.ydlcommon.bean.ShareData
import com.ydl.ydlcommon.data.PlatformDataManager
import com.ydl.ydlcommon.modular.ModularServiceManager
import com.ydl.ydlcommon.router.IYDLRouterConstant
import com.ydl.ydlcommon.router.YdlCommonRouterManager
import com.ydl.ydlcommon.utils.statusBar.StatusBarUtil
import com.ydl.ydlcommon.view.dialog.CommonDialog
import com.ydl.ydlcommon.view.dialog.YDLShareDialog
import com.yidianling.common.tools.RxImageTool
import com.yidianling.common.tools.ToastUtil
import com.yidianling.course.BuildConfig
import com.yidianling.course.CourseConstants
import com.yidianling.course.R
import com.yidianling.course.bean.COURSE_AUDIO
import com.yidianling.course.bean.CourseExtraBean
import com.yidianling.course.bean.CourseMediaBean
import com.yidianling.course.bean.CourseMediaDetailBean
import com.yidianling.course.coursePlay.presenter.AudioPlayPresenter
import com.yidianling.course.coursePlay.presenter.IAudioPlayContract
import com.yidianling.course.router.CourseIn
import com.yidianling.course.widget.CourseListDialog
import com.yidianling.course.widget.CourseListListener
import com.yidianling.course.widget.CourseSpeedDialog
import com.yidianling.ydl_pay.common.CommonPayDialog
import jp.wasabeef.glide.transformations.BlurTransformation
import kotlinx.android.synthetic.main.activity_audio_play.*
import org.json.JSONException
import org.json.JSONObject
import java.util.*


/**
 * 课程音频播放
 */
@Route(path = "/course/audioPlay")
class AudioPlayActivity : BaseMvpActivity<IAudioPlayContract.View, IAudioPlayContract.Presenter>(),
    IAudioPlayContract.View {

    private val ROUTER_PARAMS = "routerParam"
    private var mCourseId = 0 // 课程id
    private var receiverPlayUrl: String? = null // 外部选中要播放的音/视频文件地址
    private var isFromFloatView = false // 是否是悬浮窗进入
    private var from: Int = 0
    private var mCourPlayBean: CourseMediaDetailBean? = null
    private val mPlayList: ArrayList<CourseMediaBean> = ArrayList() // 播放列表
    private var mCourseSpeedDialog: CourseSpeedDialog? = null
    private var mCourseListDialog: CourseListDialog? = null
    private var mCurrentPosition = 0

    override fun turnOnImmersiveStatusBar(): Boolean = true

    override fun createPresenter(): IAudioPlayContract.Presenter = AudioPlayPresenter()

    override fun layoutResId(): Int = R.layout.activity_audio_play

    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        intent?.let {
            val courseId = it.getIntExtra("course_id", 0)
            if (0 == courseId) {
                ToastUtil.toastShort("参数错误")
                finish()
            } else if (courseId != mCourseId) {
                mCourseId = courseId
                mPresenter.getCoursePlayData(mCourseId)
            }

        }
    }

    override fun initDataAndEvent() {
        getParams()
        initView()
        mPresenter.getCoursePlayData(mCourseId)
    }

    private fun getParams() {
        intent?.let {
            if (it.hasExtra(ROUTER_PARAMS)) {
                //路由传递的入参
                val json = it.getStringExtra(ROUTER_PARAMS)
                try {
                    val jsonObject = JSONObject(json)
                    //课程ID
                    mCourseId = jsonObject.getInt(IYDLRouterConstant.EXTRA_ID)
                } catch (e: JSONException) {
                    e.printStackTrace()
                }
                return
            }
            //正常跳转所传参数
            mCourseId = it.getIntExtra("course_id", 0)
            receiverPlayUrl = it.getStringExtra("coursePlayUrl")
            isFromFloatView = it.getBooleanExtra("isFromFloatView", false)
            if (!TextUtils.isEmpty(receiverPlayUrl) && isFromFloatView)
                audio_play.mNonWifiTips = false
            try {
                if (PlayerFloatHelper.playingType == PlayTypeEnum.PLAY_TYPE_COURSE
                    && AudioPlayer.get().isPlaying
                    && TextUtils.isEmpty(receiverPlayUrl)
                ) {
                    receiverPlayUrl = AudioPlayer.get().playMusic?.path
                }
            } catch (e: Exception) {
            }
            from = it.getIntExtra("from", 0)
            if (mCourseId == 0) {
                ToastUtil.toastShort("参数错误")
                finish()
            }
        }
    }

    private fun initView() {
        iv_back.run {
            (layoutParams as? ViewGroup.MarginLayoutParams)?.topMargin = StatusBarUtil.getStatusBarHeight(this@AudioPlayActivity)
            setOnClickListener {
                onBackPressed()
            }
        }
        iv_share.setOnClickListener {
            share(mCourPlayBean?.courseExtra?.shareData)
        }
        audio_play.mListener = { playPosition ->
            togglePlaying(playPosition)
        }
    }

    //参加课程
    private fun addCourseOrder() {
        if (mCourseId == 0) {
            ToastUtil.toastShort("请退出页面重试")
            return
        }
        val userInfo = YdlCommonRouterManager.getYdlCommonRoute().getUserInfo()
        if (userInfo == null || TextUtils.isEmpty(userInfo.userId)) {
            CourseIn.loginByOneKeyLogin(mContext, true)
            return
        }

        CommonPayDialog.Build(mContext)
            .setCourseId(mCourseId.toString())
            .setToken(userInfo.token)
            .setUid(userInfo.userId)
            .setFfrom(PlatformDataManager.getRam().getChannelName())
            .setListener(object : CommonPayDialog.OnPayResultListener {
                override fun onSuccesed() {
                    mPresenter.getCoursePlayData(mCourseId)
                }

                override fun onFailed() {

                }
            })
            .setIsTestEnvironment(BuildConfig.DEBUG).build().show()
    }

    private fun togglePlaying(playPosition: Int) {
        Log.d("wjy====", "mCurrentPosition -> $mCurrentPosition   index -> $playPosition")
        if (playPosition >= mPlayList.size || playPosition < 0) {
            ToastUtil.toastShort("暂无内容")
        } else if (mCurrentPosition != playPosition) {
            Log.d("wjy====", "audio_play.play")
            mCurrentPosition = playPosition
            mCourPlayBean?.courseExtra?.let {
                if (isCanPlay(it.isBuy)) audio_play.play(mCurrentPosition)
            }
        }
    }

    private fun isCanPlay(isBuy: Boolean): Boolean {
        mPlayList.elementAtOrNull(mCurrentPosition)?.let {
            if (!it.isDemo && !isBuy) {
                if (AudioPlayer.get().isPlaying) AudioPlayer.get().pausePlayer()
                buyCourseTipDialog()
            } else if (it.mediaType == COURSE_AUDIO) {
                return true
            }
        }
        return false
    }

    private fun buyCourseTipDialog() {
        CommonDialog(this)
            .setMessage("\n购买课程，获取完整课程内容\n")
            .setLeftOnclick("放弃") {}
            .setRightClick("购买") {
                addCourseOrder() // 跳转支付页
            }
            .setCancelAble(false)
            .show()
    }

    override fun onBackPressed() {
        if (showFloatView()) {
            super.onBackPressed()
        }
    }

    private fun showFloatView(): Boolean {
        if (AudioPlayer.get().isPlaying && PlayerFloatHelper.playingType == PlayTypeEnum.PLAY_TYPE_COURSE) {
            AudioPlayer.get().playMusic?.coverPath = mCourPlayBean?.courseExtra?.pic
            AudioPlayer.get().playMusic?.artist = mCourPlayBean?.courseExtra?.doctorName

            PlayerFloatHelper.playTempData.clear()

            val hashMap = HashMap<String, String>()
            hashMap["course_id"] = mCourseId.toString()
            hashMap["media_type"] = COURSE_AUDIO.toString()
            PlayerFloatHelper.playTempData.putAll(hashMap)
        }
        return true
    }

    private fun share(share: ShareData?) {
        share?.let {
            YDLShareDialog.style1(
                this,
                share.title,
                share.share_url,
                share.desc,
                share.cover,
                "",
                ""
            ).apply {
                setCallBack(object : YDLShareDialog.ICallBack {
                    override fun callBack(type: Int) {
                        if (1 == type) {
                            ModularServiceManager.getPlatformUserService()

                            if (CourseIn.loginByOneKeyLogin(this@AudioPlayActivity, true)) {
                                //分享到心事动态
                                val url =
                                    if (TextUtils.isEmpty(share.url)) share.share_url else share.url
                                CourseIn.getDynamicService()
                                    .publishArticleToTrend(
                                        this@AudioPlayActivity,
                                        url!!,
                                        share.cover!!,
                                        share.title!!
                                    )
                            }
                        }
                    }
                })
            }.show(fragmentManager, "lose")
        } ?: let {
            ToastUtil.toastShort("缺少参数")
        }
    }

    override fun updateView(bean: CourseMediaDetailBean) {
        mCourPlayBean = bean
        mPlayList.clear()
        setPlayList(bean)
        if (!TextUtils.isEmpty(receiverPlayUrl)) {
            mCurrentPosition = mPlayList.indexOfLast {
                TextUtils.equals(receiverPlayUrl, it.url) || TextUtils.equals(
                    receiverPlayUrl!!.replace(
                        "http",
                        "https"
                    ), it.url
                )
            }
        }
        bean.courseExtra.let {
            Glide.with(this)
                .load(it.pic)
                .transition(DrawableTransitionOptions.withCrossFade())
                .apply(RequestOptions.bitmapTransform(BlurTransformation(10, 16)))
                .into(object : ViewTarget<ConstraintLayout, Drawable>(cl_bg) {
                    override fun onResourceReady(resource: Drawable, transition: Transition<in Drawable>?) {
                        view.background = resource
                    }
                })
            Glide.with(this)
                .load(it.pic)
                .transition(DrawableTransitionOptions.withCrossFade())
                .apply(RequestOptions.bitmapTransform(RoundedCorners(RxImageTool.dp2px(8f))))
                .into(iv_pic)
            GlideApp.with(this).load(it.doctorHead).error(R.drawable.course_head_place_hold_pic)
                .into(iv_icon)
            tv_title.text = it.title
            tv_name.text = it.doctorName
            tv_detail.setOnClickListener { _ -> detailClick(it) }
            iv_detail.setOnClickListener { _ -> detailClick(it) }
            tv_speed.setOnClickListener { speedClick() }
            iv_speed.setOnClickListener { speedClick() }
            tv_list.setOnClickListener { listClick() }
            iv_list.setOnClickListener { listClick() }
        }
        audio_play.setData(mCurrentPosition, convertToMusics(mPlayList), bean.courseExtra.isBuy, from)
    }

    private fun convertToMusics(list: List<CourseMediaBean>): ArrayList<Music> {
        val musics = ArrayList<Music>()
        try {
            for (i in list.indices) {
                musics.add(Music().apply {
                    path = list[i].url
                    coverPath = mCourPlayBean?.courseExtra?.pic
                    title = list[i].title
                    artist = list[i].doctorName
                })
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return musics
    }

    private fun detailClick(extra: CourseExtraBean) {
        if (TextUtils.isEmpty(extra.id)) {
            ToastUtil.toastShort("课程id错误")
        } else {
            showFloatView()
            NewH5Activity.start(
                this,
                H5Params(CourseConstants.COURSE_DETAIL_H5 + mCourseId, null)
            ) // 跳转课程H5详情
        }
    }

    private fun speedClick() {
        val audioPlay = audio_play
        showFragment(mCourseSpeedDialog ?: CourseSpeedDialog().apply {
            mCourseSpeedDialog = this
            mListener = { speed ->
                audioPlay.setSpeed(speed)
            }
        }, "courseSpeed")
    }

    private fun listClick() {
        mCourPlayBean?.courseExtra?.let {
            showFragment(mCourseListDialog?.also {
                it.setCurrentPosition(mCurrentPosition)
            } ?: CourseListDialog().also { dialog ->
                mCourseListDialog = dialog
                mCourseListDialog?.setData(it.isBuy, mCurrentPosition, mPlayList)
                dialog.mListener = object : CourseListListener {
                    override fun addCourseOrder() {
                        this@AudioPlayActivity.addCourseOrder()
                    }

                    override fun togglePlaying(position: Int) {
                        this@AudioPlayActivity.togglePlaying(position)
                    }

                }
            }, "courseList")
        }
    }

    //设置播放列表数据
    private fun setPlayList(bean: CourseMediaDetailBean) {
        if (bean.voiceSample.courseMedia.isNotEmpty()) {
            for (voiceSampleData in bean.voiceSample.courseMedia) {
                voiceSampleData.isDemo = true
                mPlayList.add(voiceSampleData)
            }
        }
        if (bean.voiceWhole.courseMedia.isNotEmpty()) {
            for (voiceCourseData in bean.voiceWhole.courseMedia) {
                mPlayList.add(voiceCourseData)
            }
        }
    }

    override fun onResume() {
        super.onResume()
        if (PlayerFloatHelper.isShow(this)) {
            PlayerFloatHelper.removeView(this)
        }
    }

    override fun onDestroy() {
        audio_play.onDestroy()
        super.onDestroy()
    }
}