package com.yidianling.course.widget

import android.content.Context
import android.os.Bundle
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.ydl.media.audio.utils.PlayProgressUtil
import com.ydl.ydlcommon.base.BaseDialogFragment
import com.ydl.ydlcommon.utils.extend.gone
import com.ydl.ydlcommon.utils.extend.invisible
import com.ydl.ydlcommon.utils.extend.visible
import com.ydl.ydlcommon.view.dialog.CommonDialog
import com.yidianling.course.R
import com.yidianling.course.bean.CourseMediaBean
import kotlinx.android.synthetic.main.dialog_course_list.*
import kotlinx.android.synthetic.main.item_course_playlist.view.*

/**
 * 课程目录弹窗
 */
class CourseListDialog : BaseDialogFragment() {

    var mListener: CourseListListener? = null
    var mCurrentPosition = 0
    private var mAdapter: CourseListDialog.CourseListAdapter? = null
    private val mCourseList = mutableListOf<CourseMediaBean>()
    private var mIsBuy = false

    override fun getLayoutResourceId(): Int = R.layout.dialog_course_list
    override fun getGravity(): Int = Gravity.BOTTOM

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        context?.let { ctx ->
            recycler_list.adapter = mAdapter ?: CourseListAdapter(ctx).also { mAdapter = it }
        }
    }

    fun setData(isBuy: Boolean, currentPosition: Int, courseList: List<CourseMediaBean>) {
        mIsBuy = isBuy
        mCurrentPosition = currentPosition
        mCourseList.clear()
        mCourseList.addAll(courseList)
//        mAdapter?.notifyDataSetChanged()
    }

    fun setCurrentPosition(currentPosition: Int) {
        mCurrentPosition = currentPosition
//        mAdapter?.notifyDataSetChanged()
    }

    inner class CourseListAdapter(
        context: Context,
        private val inflater: LayoutInflater = LayoutInflater.from(context)
    ) : RecyclerView.Adapter<CourseListAdapter.CourseListVH>() {

        inner class CourseListVH(view: View) : RecyclerView.ViewHolder(view)

        override fun onCreateViewHolder(p0: ViewGroup, itemType: Int): CourseListVH =
            CourseListVH(inflater.inflate(R.layout.item_course_playlist, p0, false))

        override fun onBindViewHolder(vh: CourseListVH, position: Int) {
            mCourseList.elementAtOrNull(position)?.let { bean ->
                vh.itemView.run {
                    if (mCurrentPosition == position) {
                        voice_play.visible()
                        voice_play.start()
                        iv_play.invisible()
//                            Glide.with(context).asGif()
//                                .load(R.drawable.course_music).into(gifView)
                    } else {
                        voice_play.gone()
                        voice_play.stop()
                        iv_play.visible()
                        if (mIsBuy) {
                            iv_play.setImageResource(R.drawable.course_play_orange)
                        } else {
                            if (bean.isDemo) {
                                iv_play.setImageResource(R.drawable.course_play_orange)
                            } else {
                                iv_play.setImageResource(R.drawable.course_play_lock)
                            }
                        }
                    }
                    tv_audition.visibility = if (bean.isDemo) View.VISIBLE else View.GONE // 试听标签
                    tv_title.text = bean.title

                    setOnClickListener {
                        if (mCurrentPosition == position) return@setOnClickListener
                        if (!bean.isDemo && !mIsBuy) {
                            CommonDialog(context)
                                .setMessage("\n购买课程，获取完整课程内容\n")
                                .setLeftOnclick("放弃") {}
                                .setRightClick("购买") {
                                    //跳转支付页
                                    mListener?.addCourseOrder()
                                }
                                .setCancelAble(false)
                                .show()
                        } else {
                            PlayProgressUtil.saveProgress(
                                context,
                                mCourseList.elementAtOrNull(position)?.url,
                                0
                            )
                            mCurrentPosition = position
                            notifyDataSetChanged()
                            mListener?.togglePlaying(position)
                            dismissAllowingStateLoss()
                        }
                    }
                }
            }
        }

        override fun getItemCount(): Int = mCourseList.size
    }

}

interface CourseListListener {
    fun addCourseOrder()
    fun togglePlaying(position: Int)
}